/***************************************************************************
 *   Copyright (C) 2007 by Sébastien Laoût                                 *
 *   slaout@linux62.org                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "themedialogs.h"
#include "themechooser.h"
#include "themeeditor.h"
#include "thememanager.h"

#include "kcolorcombo2.h"

#include <klocale.h>
#include <qlayout.h>
#include <kpushbutton.h>
#include <kiconloader.h>
#include <qlistbox.h>
#include <qdesktopwidget.h>
#include <kmessagebox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <knuminput.h>

/** class QListBoxPixmapMargin: */

QListBoxPixmapMargin::QListBoxPixmapMargin(QListBox *listbox, const QPixmap &pix, const QString &text)
 : QListBoxPixmap(listbox, pix, text)
{
}

QListBoxPixmapMargin::QListBoxPixmapMargin(QListBox *listbox, const QPixmap &pix, const QString &text, QListBoxItem *after)
 : QListBoxPixmap(listbox, pix, text, after)
{
}

int QListBoxPixmapMargin::height(const QListBox *lb) const
{
	return QListBoxPixmap::height(lb) + 2 * 2;
}

/** class ThemeChooserDialog: */

ThemeChooserDialog::ThemeChooserDialog(int screenWidth, int screenHeight, QWidget *parent)
 : KDialogBase(KDialogBase::Swallow, i18n("Choose Theme"), KDialogBase::Ok | KDialogBase::Apply | KDialogBase::Cancel,
               KDialogBase::Ok, parent, /*name=*/"ThemeChooserDialog", /*modal=*/true, /*separator=*/true),
   m_screenWidth(screenWidth), m_screenHeight(screenHeight)
{
	QWidget *page = new QWidget(this);
	QVBoxLayout *topLayout = new QVBoxLayout(page, /*margin=*/0, spacingHint());
	m_chooser = new ThemeChooserWidget(page);
	topLayout->addWidget(m_chooser);

	m_chooser->buttonNew->setIconSet(SmallIconSet("filenew"));
	m_chooser->buttonEdit->setIconSet(SmallIconSet("edit"));
	m_chooser->buttonDelete->setIconSet(SmallIconSet("editdelete"));

	ThemeList themes = ThemeManager::instance()->themes();
	int i = 0;
//	for (QValueList<Theme*>::const_iterator it = themes.begin(); it != themes.end(); ++it) {
	QPtrListIterator<Theme> it(themes);
	Theme *theme;
	while ((theme = it.current()) != 0) {
		++it;
		new QListBoxPixmapMargin(m_chooser->themeList, theme->previewPixmap(screenWidth, screenHeight), theme->themeName());
		if (theme == Theme::current())
			m_chooser->themeList->setCurrentItem(i);
		i++;
	}

	setMainWidget(page);

	QDesktopWidget desktop;
	int height = desktop.height() * 5 / 7;
	int width  = QMAX(height * 4 / 5, m_chooser->sizeHint().width() * 12 / 10);
	resize(width, height);

	connect( m_chooser->themeList, SIGNAL(doubleClicked(QListBoxItem*)), actionButton(Ok), SLOT(animateClick()) );
	connect( m_chooser->themeList, SIGNAL(returnPressed(QListBoxItem*)), actionButton(Ok), SLOT(animateClick()) );

	connect( m_chooser->buttonNew,    SIGNAL(clicked()), this, SLOT(slotNew())    );
	connect( m_chooser->buttonEdit,   SIGNAL(clicked()), this, SLOT(slotEdit())   );
	connect( m_chooser->buttonDelete, SIGNAL(clicked()), this, SLOT(slotDelete()) );
}

// TODO Gray Apply if index is same as before
// TODO Do not hide mouse when showing dialog

ThemeChooserDialog::~ThemeChooserDialog()
{
}

void ThemeChooserDialog::applyChanges()
{
	ThemeManager *manager = ThemeManager::instance();
	ThemeList themes = manager->themes();
	Theme *theme = themes.at(m_chooser->themeList->currentItem());
	manager->setTheme(theme);
}

void ThemeChooserDialog::slotApply()
{
	applyChanges();
	KDialogBase::slotApply();
}

void ThemeChooserDialog::slotOk()
{
	applyChanges();
	KDialogBase::slotOk();
}

void ThemeChooserDialog::slotNew()
{
}

void ThemeChooserDialog::slotEdit()
{
	ThemeManager *manager = ThemeManager::instance();
	ThemeList themes = manager->themes();
	Theme *theme = themes.at(m_chooser->themeList->currentItem());
	ThemeEditorDialog dialog(theme, i18n("Edit Theme"), this);
	connect( &dialog, SIGNAL(themeChanged()), this, SLOT(editedThemeChanged()) );
	dialog.exec();
}

void ThemeChooserDialog::slotDelete()
{
}

#include <iostream>
void ThemeChooserDialog::editedThemeChanged()
{
	QListBoxPixmapMargin *currentItem = (QListBoxPixmapMargin *) m_chooser->themeList->selectedItem();
	int currentIndex = m_chooser->themeList->index(currentItem);

	// Find current theme:
	ThemeManager *manager = ThemeManager::instance();
	ThemeList themes = manager->themes();
	Theme *theme = themes.at(currentIndex/*m_chooser->themeList->currentItem()*/);

std::cout << m_chooser->themeList->currentItem() << " " << ((long) theme) << " " << ((long) m_chooser->themeList->currentItem()) << std::endl;

	// Apply theme changes:
	theme->setUsed(false);
	if (theme == Theme::current())
		manager->setTheme(theme);

//	m_chooser->themeList->changeItem(theme->previewPixmap(m_screenWidth, m_screenHeight), theme->themeName(), m_chooser->themeList->currentItem());

	// Update list view:
	//QListBoxPixmapMargin *item = (QListBoxPixmapMargin *) m_chooser->themeList->selectedItem();
	QListBoxPixmapMargin *newItem = new QListBoxPixmapMargin(m_chooser->themeList, theme->previewPixmap(m_screenWidth, m_screenHeight), theme->themeName(), currentItem);
//	m_chooser->themeList->setSelected(item, false);
//	m_chooser->themeList->setCurrentItem(0);
	delete currentItem;
	m_chooser->themeList->setSelected(newItem, true);
	m_chooser->themeList->setCurrentItem(newItem);

}

/** class ThemeEditorDialog: */

ThemeEditorDialog::ThemeEditorDialog(Theme *theme, const QString &title, QWidget *parent)
 : KDialogBase(KDialogBase::Swallow, title, KDialogBase::Ok | KDialogBase::Apply | KDialogBase::Cancel,
               KDialogBase::Ok, parent, /*name=*/"ThemeChooserDialog", /*modal=*/true, /*separator=*/false),
   m_theme(theme)
{
	m_temporarTheme = new Theme("", "");
	m_theme->copyTo(m_temporarTheme);

	QWidget *page = new QWidget(this);
	QVBoxLayout *topLayout = new QVBoxLayout(page, /*margin=*/0, spacingHint());
	m_editor = new ThemeEditorWidget(page);
	topLayout->addWidget(m_editor);
	m_editor->layout()->setMargin(0);

	// Equalise Label Sizes:
	QValueList<QLabel*> labels;
	labels <<
		m_editor->themeNameLabel << m_editor->backgroundImageLabel <<
		m_editor->backgroundColorLabel << m_editor->textColorLabel << m_editor->nextPlayingColorLabel << m_editor->dateColorLabel << m_editor->hourColorLabel <<
		m_editor->progressBackgroundLabel << m_editor->progressBarLabel << m_editor->progressBackgroundTextLabel << m_editor->progressBarTextLabel;

	int maxWidth = 0;
	for (QValueList<QLabel*>::const_iterator it = labels.begin(); it != labels.end(); ++it)
		if ((*it)->sizeHint().width() > maxWidth)
			maxWidth = (*it)->sizeHint().width();

	for (QValueList<QLabel*>::const_iterator it = labels.begin(); it != labels.end(); ++it)
		(*it)->setFixedWidth(maxWidth);

	// Load Values:
	m_editor->themeName->setText(m_temporarTheme->themeName());

	m_editor->backgroundColor->setColor(m_temporarTheme->backgroundColorSetting());
	m_editor->textColor->setColor(m_temporarTheme->textColorSetting());
	m_editor->nextPlayingColor->setColor(m_temporarTheme->nextPlayingColorSetting());
	m_editor->dateColor->setColor(m_temporarTheme->dateColorSetting());
	m_editor->hourColor->setColor(m_temporarTheme->hourColorSetting());

	m_editor->progressBackgroundColor->setColor(m_temporarTheme->progressBackgroundColorSetting());
	m_editor->progressBarColor->setColor(m_temporarTheme->progressBarColorSetting());
	m_editor->progressBackgroundTextColor->setColor(m_temporarTheme->progressBackgroundTextColorSetting());
	m_editor->progressBarTextColor->setColor(m_temporarTheme->progressBarTextColorSetting());

	m_editor->backgroundColorOpacity->setValue(m_temporarTheme->backgroundColorOpacity());

	setDefaultColors();

	// Connect to Changes:
	connect( m_editor->themeName,                   SIGNAL(textChanged(const QString&)), this, SLOT(somethingChanged()) );

	connect( m_editor->backgroundColor,             SIGNAL(changed(const QColor&)),      this, SLOT(somethingChanged()) );
	connect( m_editor->textColor,                   SIGNAL(changed(const QColor&)),      this, SLOT(somethingChanged()) );
	connect( m_editor->nextPlayingColor,            SIGNAL(changed(const QColor&)),      this, SLOT(somethingChanged()) );
	connect( m_editor->dateColor,                   SIGNAL(changed(const QColor&)),      this, SLOT(somethingChanged()) );
	connect( m_editor->hourColor,                   SIGNAL(changed(const QColor&)),      this, SLOT(somethingChanged()) );

	connect( m_editor->progressBackgroundColor,     SIGNAL(changed(const QColor&)),      this, SLOT(somethingChanged()) );
	connect( m_editor->progressBarColor,            SIGNAL(changed(const QColor&)),      this, SLOT(somethingChanged()) );
	connect( m_editor->progressBackgroundTextColor, SIGNAL(changed(const QColor&)),      this, SLOT(somethingChanged()) );
	connect( m_editor->progressBarTextColor,        SIGNAL(changed(const QColor&)),      this, SLOT(somethingChanged()) );

	connect( m_editor->backgroundColorOpacity,      SIGNAL(valueChanged(int)),           this, SLOT(somethingChanged()) );

	// Finalize:
	setMainWidget(page);
}

ThemeEditorDialog::~ThemeEditorDialog()
{
}

void ThemeEditorDialog::somethingChanged()
{
	m_temporarTheme->setThemeName(m_editor->themeName->text());

	m_temporarTheme->setBackgroundColor(m_editor->backgroundColor->color());
	m_temporarTheme->setTextColor(m_editor->textColor->color());
	m_temporarTheme->setNextPlayingColor(m_editor->nextPlayingColor->color());
	m_temporarTheme->setDateColor(m_editor->dateColor->color());
	m_temporarTheme->setHourColor(m_editor->hourColor->color());

	m_temporarTheme->setProgressBackgroundColor(m_editor->progressBackgroundColor->color());
	m_temporarTheme->setProgressBarColor(m_editor->progressBarColor->color());
	m_temporarTheme->setProgressBackgroundTextColor(m_editor->progressBackgroundTextColor->color());
	m_temporarTheme->setProgressBarTextColor(m_editor->progressBarTextColor->color());

	m_temporarTheme->setBackgroundColorOpacity(m_editor->backgroundColorOpacity->value());

	setDefaultColors();
}

void ThemeEditorDialog::setDefaultColors()
{
	m_editor->backgroundColor->setDefaultColor(m_temporarTheme->defaultBackgroundColor());
	m_editor->textColor->setDefaultColor(m_temporarTheme->defaultTextColor());
	m_editor->nextPlayingColor->setDefaultColor(m_temporarTheme->defaultNextPlayingColor());
	m_editor->dateColor->setDefaultColor(m_temporarTheme->defaultDateColor());
	m_editor->hourColor->setDefaultColor(m_temporarTheme->defaultHourColor());

	m_editor->progressBackgroundColor->setDefaultColor(m_temporarTheme->defaultProgressBackgroundColor());
	m_editor->progressBarColor->setDefaultColor(m_temporarTheme->defaultProgressBarColor());
	m_editor->progressBackgroundTextColor->setDefaultColor(m_temporarTheme->defaultProgressBackgroundTextColor());
	m_editor->progressBarTextColor->setDefaultColor(m_temporarTheme->defaultProgressBarTextColor());
}

void ThemeEditorDialog::applyChanges()
{
	m_temporarTheme->copyTo(m_theme);
	m_theme->save();
	emit themeChanged();
}

void ThemeEditorDialog::slotApply()
{
	applyChanges();
	KDialogBase::slotApply();
}

void ThemeEditorDialog::slotOk()
{
	applyChanges();
	KDialogBase::slotOk();
}

#include "themedialogs.moc"
