/***************************************************************************
 *   Copyright (C) 2007 by Sébastien Laoût                                 *
 *   slaout@linux62.org                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef THEMEDIALOGS_H
#define THEMEDIALOGS_H

#include <kdialogbase.h>
#include <qlistbox.h>

class ThemeChooserWidget;
class ThemeEditorWidget;

class Theme;

class QListBoxPixmapMargin : public QListBoxPixmap
{
  public:
	QListBoxPixmapMargin(QListBox *listbox, const QPixmap &pix, const QString &text);
	QListBoxPixmapMargin(QListBox *listbox, const QPixmap &pix, const QString &text, QListBoxItem *after);
	int height(const QListBox *lb) const;
};

/**
 * @author Sébastien Laoût <slaout@linux62.org>
 */
class ThemeChooserDialog : public KDialogBase
{
  Q_OBJECT
  public:
	ThemeChooserDialog(int screenWidth, int screenHeight, QWidget *parent = 0);
	~ThemeChooserDialog();
  protected:
	void applyChanges();
  protected slots:
	void slotApply();
	void slotOk();
	void editedThemeChanged();
  private slots:
	void slotNew();
	void slotEdit();
	void slotDelete();
  private:
	ThemeChooserWidget *m_chooser;
	int m_screenWidth;
	int m_screenHeight;
};

/**
 * @author Sébastien Laoût <slaout@linux62.org>
 */
class ThemeEditorDialog : public KDialogBase
{
  Q_OBJECT
  public:
	ThemeEditorDialog(Theme *theme, const QString &title, QWidget *parent = 0);
	~ThemeEditorDialog();
  signals:
	void themeChanged();
  protected:
	void applyChanges();
  protected slots:
	void somethingChanged();
	void setDefaultColors();
	void slotApply();
	void slotOk();
  private:
	Theme *m_theme;
	Theme *m_temporarTheme;
	ThemeEditorWidget *m_editor;
};

#endif // THEMEDIALOGS_H
